##########
# .icProj project file access routines
##########
namespace eval ::ic_proj {
	variable IcProjFileName
	variable IcProjOptions
	array set IcProjOptions {}
	
	proc fatalError {errStr} {
		# stdout may not be defined like when Stage Master Explorer runs 
		if {[info exists stdout]} {
    		puts ""
    		puts "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
    		puts "!! FATAL_ERROR:"
    		puts "!!   $errStr"
    		puts "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
    		puts "Exiting on fatal error..."
    		puts ""
    		exit -1;
		}
	}
	
	proc getParmVal {parmName} {
		variable IcProjOptions
		variable IcProjFileName

		if {[info exists IcProjOptions($parmName)]} {
			return $IcProjOptions($parmName)
		} else {
			puts "WARNING:Could not find parameter $parmName in project file $IcProjFileName"
			return ""
		}
	}

	proc loadFile {fileName} {
		variable IcProjOptions
		variable IcProjFileName
		set IcProjFileName $fileName
		if {[catch { 
				set fin [open $fileName r]
				while {[gets $fin line] >= 0} {
					# looking for lines of the format: $Option "<parm>=<val>"
					# 	<val> is <parm> defined and may be a string with spaces or a space-delimited list
					if {[regexp -nocase {\$Option \"(.*?)=(.*?)\"} $line whole parm val]} {
						set IcProjOptions($parm) $val
					}
				}
			} cause]} {
			fatalError "Error while loading $fileName: $cause"
		}
	}

	proc findIcProjFile {dir} {
		set fileNames [glob -nocomplain -types f [file join $dir "*.icProj"]]
		#puts "findIcProjFile potentials:$fileNames"
		if {[llength $fileNames] > 1} {
			fatalError "Multiple *.icProj files found in $dir, please remove unused files and try again "
		} elseif {[llength $fileNames] == 0} {
			fatalError "*.icProj file not found in $dir, please be sure to run from project directory and try again "
		} else {
			return [lindex $fileNames 0]
		}
	}

	# find .icProj project file and load
	# assumes that we're executing in the project directory and there is only one .icProj file
	loadFile [findIcProjFile "."]	
	
}
