
proc impulse_elaborate { mhsinst } {
	set pcoreName [xget_hw_value $mhsinst]
	puts "!!!!!!!!!! impulse_elaborate() for $pcoreName BEGIN !!!!!!!!!!"
#	set param_names { "fpga_family" "fpga_subfamily" "fpga_partname"
#							"fpga_device" "fpga_package" "fpga_speedgrade" }
#	
#	foreach name $param_names {
#		set val [xget_hw_proj_setting $name]
#		puts "!! $name = $val"
#	}
	
	# create directory path variables
	set pcoreDataDir [xget_hw_pcore_dir $mhsinst]
	set pcoreDir [file join $pcoreDataDir ".."]
	set implementationDir [file join $pcoreDir ".." ".." "implementation"]
	set pcoreXcoDir [file join $pcoreDir "xco"]
	set pcoreCoregenDir [file join $pcoreDir "coregen"]
	set pcoreNetlistDir [file join $pcoreDir "netlist"]
	puts "!! pcore dir = $pcoreDir"
	puts "!! pcore xco dir = $pcoreXcoDir"
	puts "!! pcore coregen dir = $pcoreCoregenDir"
	puts "!! pcore netlist dir = $pcoreNetlistDir"
	
	# load support scripts
	source [file join $pcoreDataDir xco.tcl]
	source [file join $pcoreDataDir coregen.tcl]
	
	# make sure necessary directories exist
	file mkdir $pcoreCoregenDir
	file mkdir $pcoreNetlistDir
	
	# To minimize calls to CORE Generator only check for .xco files that are 
	#  in the xco directory and NOT in the coregen directory updating the xco
	#  file with device information.  Deleting the coregen directory will cause 
	#  regeneration of all xco files if the EDK project's device should change.
	#  The xco file names are assumed to be unique per core and configuration
	puts "Checking for new XCO files in xco directory..."
	set xcoFilesInXcoDirList [glob -nocomplain -directory $pcoreXcoDir -tails *.xco]
	puts "XCO files in xco directory = $xcoFilesInXcoDirList"
	set xcoFilesInCoregenDirList [glob -nocomplain -directory $pcoreCoregenDir -tails *.xco]
	puts "XCO files in coregen directory = $xcoFilesInCoregenDirList"
	
	foreach xcoF $xcoFilesInXcoDirList {
		puts -nonewline "Checking to see if $xcoF exists in coregen directory - "
		if {[lsearch $xcoFilesInCoregenDirList $xcoF] < 0} {
			puts "Does NOT exist: updating and copying from xco to coregen directory"
			array set xcoOutputParamsArr {}
			array set replaceParmsArr {}
			set replaceParmsArr(device) [xget_hw_proj_setting fpga_device]
			set replaceParmsArr(devicefamily) [xget_hw_proj_setting fpga_family]
			set replaceParmsArr(package) [xget_hw_proj_setting fpga_package]
			set replaceParmsArr(speedgrade) [xget_hw_proj_setting fpga_speedgrade]
			::xilinx::xco::setParams [file join $pcoreXcoDir $xcoF] [file join $pcoreCoregenDir $xcoF] replaceParmsArr xcoOutputParamsArr
			# make a copy of the updated XCO to be used as a CORE Generator proejct CGP file
			file copy -force [file join $pcoreCoregenDir $xcoF] [file join $pcoreCoregenDir "[file rootname $xcoF].cgp"]
		} else {
			puts "Already exists in coregen directory: not updating or copying"
		}
	}
	
	# check the timestamp of xco files in the coregen directory against their
	#  counterpart ngc files in the netlist direcotry, call CORE generator
	#  whenver xco file is newer than ngc file or ngc file doesn't exist
	puts "Checking for updated or new XCO files in coregen directory..."
	set xcoFilesInCoregenDirList [glob -nocomplain -directory $pcoreCoregenDir -tails *.xco]
	puts "XCO files in coregen directory = $xcoFilesInCoregenDirList"
	set ngcFilesInCoregenDirList [glob -nocomplain -directory $pcoreCoregenDir -tails *.ngc]
	puts "NGC files in coregen directory = $ngcFilesInCoregenDirList"
	foreach xcoF $xcoFilesInCoregenDirList {
		set ngcF "[file rootname $xcoF].ngc"
		set xcoFullFilename [file join $pcoreCoregenDir $xcoF]
		set ngcSrcFullFilename [file join $pcoreCoregenDir $ngcF]
		set ngcDestFullFilename [file join $pcoreNetlistDir $ngcF]

		puts -nonewline "Checking to see if XCO file $xcoF in xco directory has up-to-date NGC file $ngcF in coregen directory - "
		
		if {[lsearch $ngcFilesInCoregenDirList $ngcF] < 0} {
			puts "Does NOT exist: Calling CORE Generator to generate new core from xco..."
			::xilinx::coregen::generateCoreFromXco $pcoreCoregenDir "[file rootname $xcoF].cgp" $xcoF
			# "touch" generated NGC file to ensure it is newer than XCO
			catch [exec touch $ngcSrcFullFilename]
		} else {
			puts "XCO time = [file mtime $xcoFullFilename]"
			puts "NGC time = [file mtime $ngcSrcFullFilename]"
			if {[file mtime $xcoFullFilename] > [file mtime $ngcSrcFullFilename]} {
				puts "NGC file already exists in coregen directory but is older than XCO file: Regenerating..."
				::xilinx::coregen::generateCoreFromXco $pcoreCoregenDir "[file rootname $xcoF].cgp" $xcoF
				# "touch" generated NGC file to ensure it is newer than XCO 
				catch [exec touch $ngcSrcFullFilename]
			} else {				
				puts "NGC file already exists in netlist directory and is newer than XCO file: NOT regenerating"
			}
		}
		# always copy NGC file from coregen directory to netlist directory
		file copy -force $ngcSrcFullFilename $ngcDestFullFilename
		# also always copy to the implementation directory because BBD file is not always
		#  read late enough, especially when a new core is generated for the first time,
		#  to see all the NGC files to be copied to the wrapper's directory
		file copy -force $ngcDestFullFilename [file join $implementationDir $ngcF]
	}
	
	# make sure all NGC files in the netlist directory are listed in the 
	#  pcore's BBD file in the data directory - BBD is autogenerated by PSP 
	#  and always empty so just rewrite the entire file with NGCs 
	puts "Writing BBD file in data directory with NGC files present in netlist directory..."
	set ngcFilesInNetlistDirList [glob -nocomplain -directory $pcoreNetlistDir -tails *.ngc]
	set bbdFile [open [file join $pcoreDataDir "${pcoreName}_v2_1_0.bbd"] w]
	puts $bbdFile "FILES"
	set ngcFileList ""
	foreach ngcF $ngcFilesInNetlistDirList {
		append ngcFileList $ngcF ","
	}
	set ngcFileList [string trimright $ngcFileList ","]
	puts $bbdFile $ngcFileList
	close $bbdFile

	puts "!!!!!!!!!! impulse_elaborate() for $pcoreName END !!!!!!!!!!"
# stop here during intiali testing	
#	exit -1
}

