#####
# XCO file manipulation routines
#####
namespace eval ::xilinx::xco {

	# parse line of XCO file looking for parameter:
	#  SET parm=val or CSET parm=val with some or no whitespace surrounding '='
	# return: on success: list <lineBeggining> <key> <val> <lineEnding>, 
	#			 on failure: list inLine
	proc parseXcoLine { inLine } {
		if {[regexp {(^C*SET) (\S+)\s*=\s*(\S+)(.*$)} $inLine whole lBeg key val lEnd]} {
			return [list $lBeg $key $val $lEnd]
		}
		return [list $inLine]
	}
	
	# write line to XCO file fd as:
	# <beggining> <key>=<val> <ending>
	proc writeXcoLine { fd lineList } {
		set beginning [string trim [lindex $lineList 0]] 
		set key [string trim [lindex $lineList 1]]
		set val [string trim [lindex $lineList 2]]
		set ending [string trim [lindex $lineList 3]]
		puts $fd "$beginning $key=$val$ending"
	}
	
	# Take in a key-value list for replacing values in inFileName writing result
	#  to outFileName
	proc setParams { inFileName outFileName paramArrIn outputParamArrIn} {
		upvar $paramArrIn paramArr
		upvar $outputParamArrIn outputParamArr
		
		set fin [open $inFileName r]
		set fout [open $outFileName w]
		while {[gets $fin line] >= 0} {
			# pLin = list: <beggining> <key> <val> <ending>
			set pLin [parseXcoLine $line] 
			if {[llength $pLin] > 1} {
				# check to see if key is in set to change value
				set key [lindex $pLin 1]
				if {[info exists paramArr($key)]} {
					set keyVal $paramArr($key)
					lset pLin 2 $keyVal
					writeXcoLine $fout $pLin
				} else {
					# not in list, utput line as-is
					puts $fout "$line"
				}
				# store final XCO paramters for caller
				set outputParamArr([lindex $pLin 1]) [lindex $pLin 2]
			} else {
				# nothing to check, output line as-is
				puts $fout "$line"
			}
			
		}
		close $fin
		close $fout
	}
	
}

return 0