#####################################################################
# Copyright (c) 2003-2006, Impulse Accelerated Technologies, Inc.
# All Rights Reserved.
#
# export.tcl - Export a design as an EDK Core.
#
#####################################################################

proc CopySoftwareFiles {name srcdir dstdir} {
	file mkdir $dstdir/code
	
	foreach f [glob -nocomplain $srcdir/Makefile] {
		file copy -force $f $dstdir/code
	}
	foreach f [glob -nocomplain $srcdir/*.c] {
		file copy -force $f $dstdir/code
	}
	foreach f [glob -nocomplain $srcdir/*.s] {
		file copy -force $f $dstdir/code
	}
	foreach f [glob -nocomplain $srcdir/*.h] {
		file copy -force $f $dstdir/code
	}
}

proc CopyHardwareFiles {name srcdir dstdir} {
	#####
	# NOTE: Calling CORE Generator from CoDeveloper is being deprecated with move
	#        to ISE (which will call it automatically) and including a Tcl script
	#        for EDK - this was necessary to migrate to Xilinx FP v5 cores using 
	#		   autoip feature and the handling of XCO files is the model to use 
	#			for future CORE Generator IP 
	#####

	file mkdir $dstdir/pcores/impulse/hdl/vhdl
	foreach f [glob $srcdir/lib/*] {
		file copy -force $f $dstdir/pcores/impulse/hdl/vhdl
	}
	
	if { [file isdirectory $srcdir/pcore] } {
		foreach bus { apu fsl opb plb impc dvi} {
			set mpdFile $srcdir/pcore/${bus}_${name}_v2_1_0.mpd
			if { [file exists $mpdFile] } {
				set pcoreBusDir $dstdir/pcores/${bus}_[string tolower $name]_v1_00_a

				file mkdir $pcoreBusDir/data
				file copy -force $mpdFile $pcoreBusDir/data
				file copy -force $srcdir/pcore/${bus}_${name}_v2_1_0.pao $pcoreBusDir/data
				foreach f [glob -nocomplain $srcdir/pcore/*.tcl] {
					file copy -force $f $pcoreBusDir/data
				}

				file mkdir $pcoreBusDir/hdl/vhdl
				foreach f [glob -nocomplain $srcdir/pcore/*.vhd] {
					file copy -force $f $pcoreBusDir/hdl/vhdl
				}
				foreach f [glob -nocomplain $srcdir/*.vhd] {
					file copy -force $f $pcoreBusDir/hdl/vhdl
				}
				file mkdir $pcoreBusDir/hdl/verilog
				foreach f [glob -nocomplain $srcdir/pcore/*.v] {
					file copy -force $f $pcoreBusDir/hdl/verilog
				}
				foreach f [glob -nocomplain $srcdir/*.v] {
					file copy -force $f $pcoreBusDir/hdl/verilog
				}
				
				# copy bdd file generated if present
				set bddFile $srcdir/pcore/${bus}_${name}_v2_1_0.bbd
				if { [file exists $bddFile] } {
					file copy -force $bddFile $pcoreBusDir/data
				}
				# copy ngc and netlist files if present
				set coregenNgcFileList [glob -nocomplain $srcdir/netlist/*]
				if { $coregenNgcFileList != "" } {
					file mkdir $pcoreBusDir/netlist
					foreach f $coregenNgcFileList {
						file copy -force $f $pcoreBusDir/netlist
					}
				}
				
#				# copy hdl files generated for CoreGens if present
#				set coregenVhdlFileList [glob -nocomplain $srcdir/coregen/*.vhd]
#				if { $coregenVhdlFileList != "" } {
#					foreach f $coregenVhdlFileList {
#						file copy -force $f $dstdir/pcores/impulse/hdl/vhdl
#					}
#				}
				
				# copy XCO files if present
				set xcoFileList [glob -nocomplain $srcdir/xco/*.xco]
				if { $xcoFileList != "" } {
					set pcoreBusXcoDir [file join $pcoreBusDir "xco"] 
					file mkdir $pcoreBusXcoDir
					foreach f $xcoFileList {
						file copy -force $f $pcoreBusXcoDir
					}
				}
				
				# copy all Tcl files present into data directory
				set tclFileList [glob -nocomplain $srcdir/tcl/*]
				if { $tclFileList != "" } {
					set pcoreBusDataDir [file join $pcoreBusDir "data"] 
					foreach f $tclFileList {
						file copy -force $f $pcoreBusDataDir
					}
				}

				break
			}
		}
		
		foreach bus { apu fsl opb plb } {
			# Driver files not present in a platform w/o embedded CPU
			set mddFile $srcdir/driver/${bus}_${name}_v2_1_0.mdd
			if { [file exists $mddFile] } {
				set pcoreDriverDir $dstdir/drivers/${bus}_${name}_v1_00_a

				file mkdir $pcoreDriverDir/data
				file copy -force $mddFile $pcoreDriverDir/data
				file copy -force $srcdir/driver/mdd.tcl $pcoreDriverDir/data/${bus}_${name}_v2_1_0.tcl
				
				file mkdir $pcoreDriverDir/src
				foreach f [glob -nocomplain $srcdir/driver/src/*] {
					file copy -force $f $pcoreDriverDir/src
				}
				break
			}
		}
	} else {
		puts stderr "Warning: pcore not found"
	}
}

